@@ -21,10 +21,8 @@
 */
 
 #include <unistd.h>
-#include <sys/types.h>
 
 #import <Foundation/NSAutoreleasePool.h>
-#import <Foundation/NSString.h>
 #import <Foundation/NSTimeZone.h>
 
 #import <NGObjWeb/SoApplication.h>
