@@ -1,6 +1,6 @@
 /* iCalAlarm+SOGo.h - this file is part of SOGo
  *
- * Copyright (C) 2015 Inverse inc.
+ * Copyright (C) 2015-2017 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 #import <NGCards/iCalAlarm.h>
 
 @class iCalEntityObject;
+@class SOGoUser;
 
 @interface iCalAlarm (SOGoExtensions)
 
@@ -34,4 +35,6 @@
       emailAttendees: (BOOL) reminderEmailAttendees
       emailOrganizer: (BOOL) reminderEmailOrganizer;
 
+- (BOOL) userIsAttendee: (SOGoUser *) user;
+
 @end
