@@ -1,15 +1,15 @@
 /*
-  Copyright (C) 2009-2016 Inverse inc.
+  Copyright (C) 2009-2017 Inverse inc.
   Copyright (C) 2004-2005 SKYRIX Software AG
 
-  This file is part of OpenGroupware.org.
+  This file is part of SOGo.
 
-  OGo is free software; you can redistribute it and/or modify it under
+  SOGo is free software; you can redistribute it and/or modify it under
   the terms of the GNU Lesser General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.
 
-  OGo is distributed in the hope that it will be useful, but WITHOUT ANY
+  SOGo is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
   License for more details.
@@ -118,12 +118,14 @@ NSArray *SOGoMailCoreInfoKeys;
 - (NSException *) addFlags:(id)_f;
 - (NSException *) removeFlags:(id)_f;
 
-- (BOOL) isNewMail;  /* \Recent */
-- (BOOL) flagged;    /* \Flagged */
-- (BOOL) read;       /* \Unseen */
-- (BOOL) replied;    /* \Answered */
-- (BOOL) forwarded;  /* $forwarded */
-- (BOOL) deleted;    /* \Deleted */
+- (BOOL) isNewMail;   /* \Recent */
+- (BOOL) flagged;     /* \Flagged */
+- (BOOL) read;        /* \Unseen */
+- (BOOL) replied;     /* \Answered */
+- (BOOL) forwarded;   /* $forwarded */
+- (BOOL) deleted;     /* \Deleted */
+- (BOOL) isSigned;    /* S/MIME signed message */
+- (BOOL) isEncrypted; /* S/MIME encrypted message */
 
 /* deletion */
 
