@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2007-2016 Inverse inc.
+  Copyright (C) 2007-2017 Inverse inc.
   Copyright (C) 2004-2005 SKYRIX Software AG
 
   This file is part of SOGo.
@@ -37,6 +37,9 @@
 #import <NGImap4/NGImap4Envelope.h>
 #import <NGImap4/NGImap4EnvelopeAddress.h>
 #import <NGMail/NGMimeMessageParser.h>
+#import <NGMime/NGMimeMultipartBody.h>
+#import <NGMime/NGMimeType.h>
+
 
 #import <SOGo/NSArray+Utilities.h>
 #import <SOGo/NSDictionary+Utilities.h>
@@ -49,7 +52,9 @@
 
 #import "NSString+Mail.h"
 #import "NSData+Mail.h"
+#import "NSData+SMIME.h"
 #import "NSDictionary+Mail.h"
+#import "SOGoMailAccount.h"
 #import "SOGoMailFolder.h"
 #import "SOGoMailManager.h"
 #import "SOGoMailBodyPart.h"
@@ -102,9 +107,9 @@ static BOOL debugSoParts       = NO;
 
 - (void) dealloc
 {
-  [headers    release];
+  [headers release];
   [headerPart release];
-  [coreInfos  release];
+  [coreInfos release];
   [super dealloc];
 }
 
@@ -117,7 +122,8 @@ static BOOL debugSoParts       = NO;
 
 /* hierarchy */
 
-- (SOGoMailObject *)mailObject {
+- (SOGoMailObject *) mailObject
+{
   return self;
 }
 
@@ -332,11 +338,6 @@ static BOOL debugSoParts       = NO;
   return [[self fetchCoreInfos] valueForKey: @"header"];
 }
 
-- (BOOL) hasMailHeaderInCoreInfos
-{
-  return [[self mailHeaderData] length] > 0 ? YES : NO;
-}
-
 - (id) mailHeaderPart
 {
   NGMimeMessageParser *parser;
@@ -350,7 +351,7 @@ static BOOL debugSoParts       = NO;
 
   // TODO: do we need to set some delegate method which stops parsing the body?
   parser = [[NGMimeMessageParser alloc] init];
-  headerPart = [[parser parsePartFromData:data] retain];
+  headerPart = [[parser parsePartFromData: data] retain];
   [parser release]; parser = nil;
 
   if (headerPart == nil) {
@@ -400,7 +401,8 @@ static BOOL debugSoParts       = NO;
   // application/pkcs7-mime
   // application/pdf
   // etc.
-  if ([[[info valueForKey: @"type"] lowercaseString] isEqualToString: @"application"])
+  if ([[[info valueForKey: @"type"] lowercaseString] isEqualToString: @"application"] ||
+      [[[info valueForKey: @"type"] lowercaseString] isEqualToString: @"audio"])
     return info;
 
   /*
@@ -785,8 +787,8 @@ static BOOL debugSoParts       = NO;
   if ([prefix hasSuffix: @"/"])
     prefix = [prefix substringToIndex: [prefix length] - 1];
   [self _feedFileAttachmentIds: attachmentIds
-	withInfos: [coreInfos objectForKey: @"bodystructure"]
-	andPrefix: prefix];
+                     withInfos: [coreInfos objectForKey: @"bodystructure"]
+                     andPrefix: prefix];
 
   return attachmentIds;
 }
@@ -810,22 +812,7 @@ static BOOL debugSoParts       = NO;
 
   if (!filename)
     {
-      // We might end up here because of MUA that actually strips the
-      // Content-Disposition (and thus, the filename) when mails containing
-      // attachments have been forwarded. Thunderbird (2.x) does just that
-      // when forwarding mails with images attached to them (using cid:...).
-      if ([mimeType hasPrefix: @"application/"] ||
-	  [mimeType hasPrefix: @"audio/"] ||
-	  [mimeType hasPrefix: @"image/"] ||
-	  [mimeType hasPrefix: @"video/"])
-      {
-          filename = [NSString stringWithFormat: @"unknown_%@", path];
-      }
-      else
-      {
-        if ([mimeType isEqualToString: @"message/rfc822"])
-          filename = [NSString stringWithFormat: @"email_%@.eml", path];
-      }
+      filename = [mimeType asPreferredFilenameUsingPath: path];
     }
 
   if (filename)
@@ -885,7 +872,7 @@ static BOOL debugSoParts       = NO;
         {
           path = @"1";
 
-          // We set the path to 0 in case of a smime mail if not provided.
+          // We set the path to 0 in case of a S/MIME mail if not provided.
           subtype = [[part objectForKey: @"subtype"] lowercaseString];
           if ([subtype isEqualToString: @"pkcs7-mime"] || [subtype isEqualToString: @"x-pkcs7-mime"])
              path = @"0";
@@ -913,7 +900,9 @@ static BOOL debugSoParts       = NO;
 
   keys = [NSMutableArray array];
   [self _fetchFileAttachmentKeysInPart: [self bodyStructure]
-                             intoArray: keys withPath: nil andPrefix: prefix];
+                             intoArray: keys
+                              withPath: nil
+                             andPrefix: prefix];
 
   return keys;
 }
@@ -1183,11 +1172,35 @@ static BOOL debugSoParts       = NO;
 		    inContext: (id) _ctx
 {
   // TODO: we might want to check for existence prior controller creation
-  Class clazz;
-  NSArray *parts;
-  int part;
   NSDictionary *partDesc;
   NSString *mimeType;
+  NSArray *parts;
+  Class clazz;
+
+  int part;
+
+  if ([self isEncrypted])
+    {
+      NSData *certificate;
+
+      certificate = [[self mailAccountFolder] certificate];
+
+      // If we got a user certificate, let's use it. Otherwise we fallback
+      // to the current parts fetching code.
+      if (certificate)
+        {
+          NGMimeMessage *m;
+          id part;
+
+          m = [[self content] messageFromEncryptedDataAndCertificate: certificate];
+
+          part = [[[m body] parts] objectAtIndex: ([_key intValue]-1)];
+          mimeType = [[part contentType] stringValue];
+          clazz = [SOGoMailBodyPart bodyPartClassForMimeType: mimeType
+                                                   inContext: _ctx];
+          return [clazz objectWithName:_key inContainer: self];
+        }
+    }
 
   parts = [[self bodyStructure] objectForKey: @"parts"];
 
@@ -1410,7 +1423,7 @@ static BOOL debugSoParts       = NO;
 - (NSException *) moveToFolderNamed: (NSString *) folderName
                           inContext: (id)_ctx
 {
-  NSException    *error;
+  NSException *error;
 
   if (![self copyToFolderNamed: folderName
 	     inContext: _ctx])
@@ -1709,6 +1722,39 @@ static BOOL debugSoParts       = NO;
   return [self _hasFlag: @"deleted"];
 }
 
+- (BOOL) isSigned
+{
+  NSString *type, *subtype, *protocol;
+  NGMimeType *contentType;
+
+  contentType = [[self mailHeaders] objectForKey: @"content-type"];
+  type = [[contentType type] lowercaseString];
+  subtype = [[contentType subType] lowercaseString];
+  protocol = [[contentType valueOfParameter: @"protocol"] lowercaseString];
+
+  return ([type isEqualToString: @"multipart"] &&
+          [subtype isEqualToString: @"signed"] &&
+          ([protocol isEqualToString: @"application/x-pkcs7-signature"] ||
+           [protocol isEqualToString: @"application/pkcs7-signature"]));
+}
+
+- (BOOL) isEncrypted
+{
+  NSString *type, *subtype;
+
+  type = [[[[self mailHeaders] objectForKey: @"content-type"] type] lowercaseString];
+  subtype = [[[[self mailHeaders] objectForKey: @"content-type"] subType] lowercaseString];
+
+  if ([type isEqualToString: @"application"])
+    {
+      if ([subtype isEqualToString: @"x-pkcs7-mime"] ||
+          [subtype isEqualToString: @"pkcs7-mime"])
+        return YES;
+    }
+
+  return NO;
+}
+
 - (NSString *) textDescription
 {
 #warning We should send the content as an NSData
