@@ -19,8 +19,9 @@
    02111-1307, USA.
 */
 
-#import <Foundation/NSCalendarDate.h>
 #import <Foundation/NSTimeZone.h>
+#import <Foundation/NSValue.h>
+
 #import <NGExtensions/NSCalendarDate+misc.h>
 
 #import "NSCalendarDate+SOGo.h"
@@ -88,9 +89,9 @@ static NSString *rfc822Months[] = {@"", @"Jan", @"Feb", @"Mar", @"Apr",
   NSString *str;
 
   str = [NSString stringWithFormat: @"%.4d%.2d%.2d",
-                  [self yearOfCommonEra],
-                  [self monthOfYear],
-                  [self dayOfMonth]];
+                  (int)[self yearOfCommonEra],
+                  (int)[self monthOfYear],
+                  (int)[self dayOfMonth]];
 
   return str;
 }
@@ -107,12 +108,41 @@ static NSString *rfc822Months[] = {@"", @"Jan", @"Feb", @"Mar", @"Apr",
 
   return
     [NSString stringWithFormat: @"%@, %.2d %@ %d %.2d:%.2d:%.2d %+.4d",
-	      rfc822Days[[self dayOfWeek]], [self dayOfMonth],
-	      rfc822Months[[self monthOfYear]], [self yearOfCommonEra],
-	      [self hourOfDay], [self minuteOfHour], [self secondOfMinute],
+	      rfc822Days[[self dayOfWeek]], (int)[self dayOfMonth],
+	      rfc822Months[[self monthOfYear]], (int)[self yearOfCommonEra],
+	      (int)[self hourOfDay], (int)[self minuteOfHour], (int)[self secondOfMinute],
 	      timeZoneShift];
 }
 
+- (NSString *) iso8601DateString
+{
+  NSNumber *day, *month, *year, *hour, *minute;
+  int timeZoneHourShift, timeZoneMinuteShift, tzSeconds;
+  char buf[23];
+
+  day = [NSNumber numberWithInt: [self dayOfMonth]];
+  month = [NSNumber numberWithInt: [self monthOfYear]];
+  year = [NSNumber numberWithInt: [self yearOfCommonEra]];
+  hour = [NSNumber numberWithInt: [self hourOfDay]];
+  minute = [NSNumber numberWithInt: [self minuteOfHour]];
+  memset(buf, 0, 23);
+
+  tzSeconds = [[self timeZone] secondsFromGMT];
+  timeZoneHourShift = (tzSeconds / 3600);
+  tzSeconds -= timeZoneHourShift * 3600;
+  timeZoneMinuteShift = tzSeconds / 60;
+
+  snprintf(buf, 23, "%04d-%02d-%02dT%02d:%02d%+.2d:%02d",
+           [year intValue],
+           [month intValue],
+           [day intValue],
+           [hour intValue],
+           [minute intValue],
+           timeZoneHourShift,
+           timeZoneMinuteShift);
+
+  return [NSString stringWithCString: buf];
+}
 
 #define secondsOfDistantFuture 1073741823.0
 #define secondsOfDistantPast -1073741823.0
