@@ -1,9 +1,8 @@
 /* SQLSource.h - this file is part of SOGo
  *
- * Copyright (C) 2009-2012 Inverse inc.
+ * Copyright (C) 2009-2017 Inverse inc.
  *
- * Authors: Ludovic Marcotte <lmarcotte@inverse.ca>
- *          Francis Lachapelle <flachapelle@inverse.ca>
+ * This file is part of SOGo.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,6 +34,7 @@
 
 #import <SOGo/SOGoSystemDefaults.h>
 
+#import "NSArray+Utilities.h"
 #import "NSString+Utilities.h"
 #import "NSString+Crypto.h"
 
@@ -57,7 +57,7 @@
  *
  * A SQL source can be defined like this:
  *
- *  {   
+ *  {
  *    id = zot;
  *    type = sql;
  *    viewURL = "mysql://sogo:sogo@127.0.0.1:5432/sogo/sogo_view";
@@ -90,12 +90,17 @@
       _authenticationFilter = nil;
       _loginFields = nil;
       _mailFields = nil;
+      // "mail" expands to all entries of MailFieldNames
+      _searchFields = [NSArray arrayWithObjects: @"c_cn", @"mail", nil];
+      [_searchFields retain];
       _userPasswordAlgorithm = nil;
       _viewURL = nil;
       _kindField = nil;
       _multipleBookingsField = nil;
       _imapHostField = nil;
       _sieveHostField = nil;
+      _listRequiresDot = YES;
+      _modulesConstraints = nil;
     }
 
   return self;
@@ -107,6 +112,7 @@
   [_authenticationFilter release];
   [_loginFields release];
   [_mailFields release];
+  [_searchFields release];
   [_userPasswordAlgorithm release];
   [_viewURL release];
   [_kindField release];
@@ -114,13 +120,16 @@
   [_domainField release];
   [_imapHostField release];
   [_sieveHostField release];
-   
+  [_modulesConstraints release];
+
   [super dealloc];
 }
 
 - (id) initFromUDSource: (NSDictionary *) udSource
                inDomain: (NSString *) sourceDomain
 {
+  NSNumber *dotValue;
+
   self = [self init];
 
   ASSIGN(_sourceID, [udSource objectForKey: @"id"]);
@@ -134,20 +143,27 @@
   ASSIGN(_kindField, [udSource objectForKey: @"KindFieldName"]);
   ASSIGN(_multipleBookingsField, [udSource objectForKey: @"MultipleBookingsFieldName"]);
   ASSIGN(_domainField, [udSource objectForKey: @"DomainFieldName"]);
+  ASSIGN(_modulesConstraints, [udSource objectForKey: @"ModulesConstraints"]);
+  if ([udSource objectForKey: @"SearchFieldNames"])
+    ASSIGN(_searchFields, [udSource objectForKey: @"SearchFieldNames"]);
   if ([udSource objectForKey: @"prependPasswordScheme"])
     _prependPasswordScheme = [[udSource objectForKey: @"prependPasswordScheme"] boolValue];
   else
     _prependPasswordScheme = NO;
-  
+
   if (!_userPasswordAlgorithm)
     _userPasswordAlgorithm = @"none";
 
   if ([udSource objectForKey: @"viewURL"])
     _viewURL = [[NSURL alloc] initWithString: [udSource objectForKey: @"viewURL"]];
 
+  dotValue = [udSource objectForKey: @"listRequiresDot"];
+  if (dotValue)
+    [self setListRequiresDot: [dotValue boolValue]];
+
 #warning this domain code has no effect yet
   if ([sourceDomain length])
-    ASSIGN (_domain, sourceDomain);
+    ASSIGN(_domain, sourceDomain);
 
   if (!_viewURL)
     {
@@ -163,6 +179,11 @@
   return _domain;
 }
 
+- (NSArray *) searchFields
+{
+  return _searchFields;
+}
+
 - (BOOL) _isPassword: (NSString *) plainPassword
              equalTo: (NSString *) encryptedPassword
 {
@@ -183,7 +204,7 @@
 {
   NSString *pass;
   NSString* result;
-  
+
   pass = [plainPassword asCryptedPassUsingScheme: _userPasswordAlgorithm];
 
   if (pass == nil)
@@ -192,13 +213,6 @@
       return nil;
     }
 
-  if ([_userPasswordAlgorithm caseInsensitiveCompare: @"md5-crypt"] == NSOrderedSame ||
-      [_userPasswordAlgorithm caseInsensitiveCompare: @"sha256-crypt"] == NSOrderedSame ||
-      [_userPasswordAlgorithm caseInsensitiveCompare: @"sha512-crypt"] == NSOrderedSame)
-    {
-      _userPasswordAlgorithm = @"crypt";
-    }
-
   if (_prependPasswordScheme)
     result = [NSString stringWithFormat: @"{%@}%@", _userPasswordAlgorithm, pass];
   else
@@ -264,7 +278,7 @@
                              @" WHERE ",
                              [_viewURL gcsTableName]];
       if (_authenticationFilter)
-        {          
+        {
           qualifier = [[EOAndQualifier alloc] initWithQualifiers:
                                                 qualifier,
                        [EOQualifier qualifierWithQualifierFormat: _authenticationFilter],
@@ -272,18 +286,18 @@
           [qualifier autorelease];
         }
       [qualifier _gcsAppendToString: sql];
-      
+
       ex = [channel evaluateExpressionX: sql];
       if (!ex)
         {
           NSDictionary *row;
           NSArray *attrs;
           NSString *value;
-          
+
           attrs = [channel describeResults: NO];
           row = [channel fetchAttributes: attrs  withZone: NULL];
           value = [row objectForKey: @"c_password"];
-      
+
           rc = [self _isPassword: _pwd  equalTo: value];
 	  [channel cancelFetch];
         }
@@ -308,9 +322,9 @@
  * @return YES if the password was successfully changed.
  */
 - (BOOL) changePasswordForLogin: (NSString *) login
-		    oldPassword: (NSString *) oldPassword
-		    newPassword: (NSString *) newPassword
-			   perr: (SOGoPasswordPolicyError *) perr
+	    oldPassword: (NSString *) oldPassword
+	    newPassword: (NSString *) newPassword
+	   perr: (SOGoPasswordPolicyError *) perr
 {
   EOAdaptorChannel *channel;
   GCSChannelManager *cm;
@@ -339,10 +353,10 @@
       if (channel)
 	{
 	  sqlstr = [NSString stringWithFormat: (@"UPDATE %@"
-						@" SET c_password = '%@'"
-						@" WHERE c_uid = '%@'"),
-			     [_viewURL gcsTableName], encryptedPassword, login];
-	  
+	@" SET c_password = '%@'"
+	@" WHERE c_uid = '%@'"),
+	     [_viewURL gcsTableName], encryptedPassword, login];
+
 	  ex = [channel evaluateExpressionX: sqlstr];
 	  if (!ex)
 	    {
@@ -355,13 +369,14 @@
 	  [cm releaseChannel: channel];
 	}
     }
-  
+
   return didChange;
 }
 
+/*
 - (NSString *) _whereClauseFromArray: (NSArray *) theArray
                                value: (NSString *) theValue
-			       exact: (BOOL) theBOOL
+	       exact: (BOOL) theBOOL
 {
   NSMutableString *s;
   int i;
@@ -378,6 +393,36 @@
 
   return s;
 }
+*/
+
+- (void) _fillConstraintsForModule: (NSString *) module
+                        intoRecord: (NSMutableDictionary *) record
+{
+  NSDictionary *constraints;
+  NSEnumerator *matches;
+  NSString *currentMatch, *currentValue, *recordValue;
+  BOOL result;
+
+  result = YES;
+
+  constraints = [_modulesConstraints objectForKey: module];
+  if (constraints)
+    {
+      matches = [[constraints allKeys] objectEnumerator];
+      while (result == YES && (currentMatch = [matches nextObject]))
+        {
+          currentValue = [constraints objectForKey: currentMatch];
+          recordValue = [record objectForKey: currentMatch];
+          result = NO;
+
+          if ([recordValue caseInsensitiveMatches: currentValue])
+            result = YES;
+        }
+    }
+
+  [record setObject: [NSNumber numberWithBool: result]
+             forKey: [NSString stringWithFormat: @"%@Access", module]];
+}
 
 - (NSDictionary *) _lookupContactEntry: (NSString *) theID
                          considerEmail: (BOOL) b
@@ -425,7 +470,7 @@
                 }
             }
         }
-      
+
       domainQualifier = nil;
       if (_domainField && domain)
         {
@@ -443,7 +488,7 @@
                                                               value: [theID lowercaseString]];
           [loginQualifier autorelease];
           [qualifiers addObject: loginQualifier];
-          
+
 	  if (_mailFields)
 	    {
               for (i = 0; i < [_mailFields count]; i++)
@@ -461,7 +506,7 @@
                 }
             }
 	}
-      
+
       sql = [NSMutableString stringWithFormat: @"SELECT *"
                              @" FROM %@"
                              @" WHERE ",
@@ -491,12 +536,9 @@
                              forKey: [field substringFromIndex: 2]];
             }
 
-          // FIXME
-          // We have to do this here since we do not manage modules
-          // constraints right now over a SQL backend.
-          [response setObject: [NSNumber numberWithBool: YES] forKey: @"CalendarAccess"];
-          [response setObject: [NSNumber numberWithBool: YES] forKey: @"MailAccess"];
-          [response setObject: [NSNumber numberWithBool: YES] forKey: @"ActiveSyncAccess"];
+          [self _fillConstraintsForModule: @"Calendar"    intoRecord: response];
+          [self _fillConstraintsForModule: @"Mail"        intoRecord: response];
+          [self _fillConstraintsForModule: @"ActiveSync"  intoRecord: response];
 
 	  // We set the domain, if any
           value = nil;
@@ -510,7 +552,7 @@
 
 	  // We populate all mail fields
 	  emails = [NSMutableArray array];
-	  
+
 	  if ([response objectForKey: @"mail"])
 	    [emails addObject: [response objectForKey: @"mail"]];
 
@@ -520,12 +562,13 @@
 	      int i;
 
 	      for (i = 0; i < [_mailFields count]; i++)
-		if ((s = [response objectForKey: [_mailFields objectAtIndex: i]]) &&
-		    [[s stringByTrimmingSpaces] length] > 0)
-		  [emails addObjectsFromArray: [s componentsSeparatedByString: @" "]];
+	if ((s = [response objectForKey: [_mailFields objectAtIndex: i]]) &&
+                    [s isNotNull] &&
+	    [[s stringByTrimmingSpaces] length] > 0)
+	  [emails addObjectsFromArray: [s componentsSeparatedByString: @" "]];
 	    }
-	  
-	  [response setObject: emails  forKey: @"c_emails"];
+
+	  [response setObject: [emails uniqueObjects]  forKey: @"c_emails"];
           if (_imapHostField)
             {
               value = [response objectForKey: _imapHostField];
@@ -575,7 +618,7 @@
             }
           else
             [response setObject: [NSNumber numberWithBool: YES] forKey: @"canAuthenticate"];
-        
+
           // We check if we should use a different login for IMAP
           if (_imapLoginField)
             {
@@ -585,26 +628,26 @@
 
 	  // We check if it's a resource of not
 	  if (_kindField)
-	    {	      
+	    {
 	      if ((value = [response objectForKey: _kindField]) && [value isNotNull])
-		{
-		  if ([value caseInsensitiveCompare: @"location"] == NSOrderedSame ||
-		      [value caseInsensitiveCompare: @"thing"] == NSOrderedSame ||
-		      [value caseInsensitiveCompare: @"group"] == NSOrderedSame) 
-		    {
-		      [response setObject: [NSNumber numberWithInt: 1]
-				forKey: @"isResource"];
-		    }
-		}
+	{
+	  if ([value caseInsensitiveCompare: @"location"] == NSOrderedSame ||
+	      [value caseInsensitiveCompare: @"thing"] == NSOrderedSame ||
+	      [value caseInsensitiveCompare: @"group"] == NSOrderedSame)
+	    {
+	      [response setObject: [NSNumber numberWithInt: 1]
+	forKey: @"isResource"];
+	    }
+	}
 	    }
 
 	  if (_multipleBookingsField)
 	    {
 	      if ((value = [response objectForKey: _multipleBookingsField]))
-		{
-		  [response setObject: [NSNumber numberWithInt: [value intValue]]
-			    forKey: @"numberOfSimultaneousBookings"];
-		}
+	{
+	  [response setObject: [NSNumber numberWithInt: [value intValue]]
+	    forKey: @"numberOfSimultaneousBookings"];
+	}
 	    }
 
           [response setObject: self forKey: @"source"];
@@ -729,7 +772,7 @@
           NSString *value;
 
           attrs = [channel describeResults: NO];
-          
+
           while ((row = [channel fetchAttributes: attrs withZone: NULL]))
             {
               value = [row objectForKey: @"c_uid"];
@@ -745,7 +788,7 @@
     [self errorWithFormat:@"failed to acquire channel for URL: %@",
           [_viewURL absoluteString]];
 
-  
+
   return results;
 }
 
@@ -755,81 +798,104 @@
 }
 
 - (NSArray *) fetchContactsMatching: (NSString *) filter
+                       withCriteria: (NSArray *) criteria
                            inDomain: (NSString *) domain
 {
   EOAdaptorChannel *channel;
-  NSMutableArray *results;
+  NSEnumerator *criteriaList;
+  NSMutableArray *fields, *results;
   GCSChannelManager *cm;
   NSException *ex;
   NSMutableString *sql;
-  NSString *lowerFilter;
-  
+  NSString *lowerFilter, *filterFormat, *currentCriteria, *qs;
+
   results = [NSMutableArray array];
 
-  cm = [GCSChannelManager defaultChannelManager];
-  channel = [cm acquireOpenChannelForURL: _viewURL];
-  if (channel)
+  if ([filter length] > 0 || !_listRequiresDot)
     {
-      lowerFilter = [filter lowercaseString];
-      lowerFilter = [lowerFilter stringByReplacingString: @"'"  withString: @"''"];
-
-      sql = [NSMutableString stringWithFormat: (@"SELECT *"
-                                         @" FROM %@"
-                                         @" WHERE"
-                                         @" (LOWER(c_cn) LIKE '%%%@%%'"
-                                         @" OR LOWER(mail) LIKE '%%%@%%'"),
-                      [_viewURL gcsTableName],
-                      lowerFilter, lowerFilter];
-      
-      if (_mailFields && [_mailFields count] > 0)
+      cm = [GCSChannelManager defaultChannelManager];
+      channel = [cm acquireOpenChannelForURL: _viewURL];
+      if (channel)
         {
-          [sql appendString: [self _whereClauseFromArray: _mailFields  value: lowerFilter  exact: NO]];
-        }
-
-      [sql appendString: @")"];
-
-      if (_domainField)
-        {
-          if ([domain length])
+          fields = [NSMutableArray array];
+          if ([filter length])
             {
-              EOQualifier *domainQualifier;
-              domainQualifier =
-                [self _visibleDomainsQualifierFromDomain: domain];
-              if (domainQualifier)
+              lowerFilter = [filter lowercaseString];
+              lowerFilter = [lowerFilter asSafeSQLString];
+              filterFormat = [NSString stringWithFormat: @"LOWER(%%@) LIKE '%%%%%@%%%%'", lowerFilter];
+              if (criteria)
+                criteriaList = [criteria objectEnumerator];
+              else
+                criteriaList = [[self searchFields] objectEnumerator];
+
+              while (( currentCriteria = [criteriaList nextObject] ))
                 {
-                  [sql appendFormat: @" AND ("];
-                  [domainQualifier _gcsAppendToString: sql];
-                  [sql appendFormat: @")"];
+                  if ([currentCriteria isEqualToString: @"mail"])
+                    {
+                      // Expand to all mail fields
+                      [fields addObject: currentCriteria];
+                      if (_mailFields)
+                        [fields addObjectsFromArray: _mailFields];
+                    }
+                  else if ([[self searchFields] containsObject: currentCriteria])
+                    [fields addObject: currentCriteria];
+                }
+            }
+
+          sql = [NSMutableString stringWithFormat: @"SELECT * FROM %@ WHERE (", [_viewURL gcsTableName]];
+
+          if ([fields count])
+            {
+              qs = [[[fields uniqueObjects] stringsWithFormat: filterFormat] componentsJoinedByString: @" OR "];
+              [sql appendString: qs];
+            }
+          else
+            [sql appendString: @"1 = 1"];
+          [sql appendString: @")"];
+
+          if (_domainField)
+            {
+              if ([domain length])
+                {
+                  EOQualifier *domainQualifier;
+                  domainQualifier = [self _visibleDomainsQualifierFromDomain: domain];
+                  if (domainQualifier)
+                    {
+                      [sql appendFormat: @" AND ("];
+                      [domainQualifier _gcsAppendToString: sql];
+                      [sql appendFormat: @")"];
+                    }
+                }
+              else
+                [sql appendFormat: @" AND %@ IS NULL", _domainField];
+            }
+
+          ex = [channel evaluateExpressionX: sql];
+          if (!ex)
+            {
+              NSDictionary *row;
+              NSMutableDictionary *mutableRow;
+              NSArray *attrs;
+
+              attrs = [channel describeResults: NO];
+
+              while ((row = [channel fetchAttributes: attrs withZone: NULL]))
+                {
+                  mutableRow = [row mutableCopy];
+                  [mutableRow setObject: self forKey: @"source"];
+                  [results addObject: mutableRow];
+                  [mutableRow release];
                 }
             }
           else
-            [sql appendFormat: @" AND %@ IS NULL", _domainField];
-        }
-
-      ex = [channel evaluateExpressionX: sql];
-      if (!ex)
-        {
-          NSDictionary *row;
-          NSArray *attrs;
-
-          attrs = [channel describeResults: NO];
-
-          while ((row = [channel fetchAttributes: attrs withZone: NULL]))
-            {
-              row = [row mutableCopy];
-              [(NSMutableDictionary *) row setObject: self forKey: @"source"];
-              [results addObject: row];
-              [row release];
-            }
+            [self errorWithFormat: @"could not run SQL '%@': %@", sql, ex];
+          [cm releaseChannel: channel];
         }
       else
-        [self errorWithFormat: @"could not run SQL '%@': %@", sql, ex];
-      [cm releaseChannel: channel];
+        [self errorWithFormat:@"failed to acquire channel for URL: %@",
+              [_viewURL absoluteString]];
     }
-  else
-    [self errorWithFormat:@"failed to acquire channel for URL: %@",
-          [_viewURL absoluteString]];
-  
+
   return results;
 }
 
@@ -855,13 +921,12 @@
 
 - (void) setListRequiresDot: (BOOL) newListRequiresDot
 {
+  _listRequiresDot = newListRequiresDot;
 }
 
 - (BOOL) listRequiresDot
 {
-  /* This method is not implemented for SQLSource. It must enable a mechanism
-     where using "." is not required to list the content of addressbooks. */
-  return YES;
+  return _listRequiresDot;
 }
 
 /* card editing */
