@@ -18,10 +18,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#import <Foundation/NSArray.h>
 #import <Foundation/NSDictionary.h>
 
-#import <NGObjWeb/SoObjectRequestHandler.h>
 #import <NGObjWeb/WOApplication.h>
 #import <NGObjWeb/WEClientCapabilities.h>
 #import <NGObjWeb/WORequest+So.h>
@@ -180,14 +178,21 @@
   cc = [self clientCapabilities];
 
   b = (
-       [[cc userAgent] rangeOfString: @"CFNetwork"].location != NSNotFound
-       && [[cc userAgent] rangeOfString: @"Darwin"].location != NSNotFound
-       || (
-           [[cc userAgent] rangeOfString: @"CFNetwork"].location != NSNotFound
-            || [[cc userAgent] rangeOfString: @"Mac OS X"].location != NSNotFound
-           )
-       && [[cc userAgent] rangeOfString: @"AddressBook"].location != NSNotFound
-       );
+        (
+          [[cc userAgent] rangeOfString: @"CFNetwork"].location != NSNotFound
+          && [[cc userAgent] rangeOfString: @"Darwin"].location != NSNotFound
+        )
+        ||
+        (
+          [[cc userAgent] rangeOfString: @"CFNetwork"].location != NSNotFound
+          && [[cc userAgent] rangeOfString: @"AddressBook"].location != NSNotFound
+        )
+        ||
+        (
+          [[cc userAgent] rangeOfString: @"Mac OS X"].location != NSNotFound
+          && [[cc userAgent] rangeOfString: @"AddressBook"].location != NSNotFound
+        )
+      );
 
   return b;
 }
