@@ -30,6 +30,7 @@
 #import <NGMail/NGMimeMessage.h>
 
 #import <NGCards/iCalAlarm.h>
+#import <NGCards/iCalEvent.h>
 
 #import <SOGo/NSCalendarDate+SOGo.h>
 #import <SOGo/NSString+Utilities.h>
@@ -37,9 +38,20 @@
 #import <SOGo/SOGoMailer.h>
 #import <SOGo/SOGoProductLoader.h>
 #import <SOGo/SOGoUser.h>
+#import <SOGo/SOGoUserManager.h>
+#import <SOGo/NSDictionary+Utilities.m>
+#import <SOGo/NSObject+Utilities.h>
+#import <SOGo/SOGoUserFolder.h>
 #import <Appointments/iCalEntityObject+SOGo.h>
 #import <Appointments/iCalPerson+SOGo.h>
 #import <Appointments/SOGoEMailAlarmsManager.h>
+#import <Appointments/SOGoAppointmentFolder.h>
+#import <Appointments/SOGoAppointmentFolders.h>
+
+#import <NGObjWeb/WOApplication.h>
+#import <NGObjWeb/WOContext+SoObjects.h>
+#import <Appointments/SOGoAptMailReminder.h>
+#import <WEExtensions/WEResourceManager.h>
 
 #import "SOGoEAlarmsNotifier.h"
 
@@ -81,14 +93,14 @@
 
 - (NGMutableHashMap *) _headersForAlarm: (iCalAlarm *) alarm
                               withOwner: (SOGoUser *) owner
+                            withSubject: (NSString *) subject
 {
   NGMutableHashMap *headers;
-  NSString *dateString, *subject, *fullName, *email;
+  NSString *dateString, *fullName, *email;
   NSDictionary *identity;
 
   headers = [NGMutableHashMap hashMap];
 
-  subject = [[alarm summary] asQPSubjectString: @"utf-8"];
   [headers addObject: subject forKey: @"Subject"];
   dateString = [[NSCalendarDate date] rfc822DateString];
   [headers addObject: dateString forKey: @"Date"];
@@ -96,11 +108,11 @@
   [headers addObject: @"SOGo Alarms Notifier/1.0" forKey: @"User-Agent"];
   [headers addObject: @"high" forKey: @"Importance"];
   [headers addObject: @"1" forKey: @"X-Priority"];
-  [headers setObject: @"text/plain; charset=\"utf-8\"" forKey: @"Content-Type"];
+  [headers setObject: @"text/html; charset=\"utf-8\"" forKey: @"Content-Type"];
   [headers setObject: @"quoted-printable" forKey: @"Content-Transfer-Encoding"];
 
   identity = [owner primaryIdentity];
-  fullName = [[identity objectForKey: @"fullName"] asQPSubjectString: @"utf-8"];
+  fullName = [identity objectForKey: @"fullName"];
   if ([fullName length])
     email = [NSString stringWithFormat: @"%@ <%@>", fullName,
                 [identity objectForKey: @"email"]];
@@ -113,58 +125,87 @@
 
 - (void) _sendMessageWithHeaders: (NGMutableHashMap *) headers
                          content: (NSData *) content
-                      toAttendee: (iCalPerson *) attendee
-                            from: (NSString *) from
+                              to: (NSDictionary *) to
                       withMailer: (SOGoMailer *) mailer
 {
   NGMimeMessage *message;
-  NSString *to, *headerTo, *attendeeName;
+  NSString *headerTo;
 
-  attendeeName = [[attendee cnWithoutQuotes] asQPSubjectString: @"utf-8"];
-  if ([attendeeName length])
-    headerTo = [NSString stringWithFormat: @"%@ <%@>", attendeeName,
-                         [attendee rfc822Email]];
-  else
-    headerTo = [attendee rfc822Email];
+  headerTo = [NSString stringWithFormat: @"%@ <%@>", [to objectForKey: @"fullName"],
+                       [to objectForKey: @"email"]];
   [headers setObject: headerTo forKey: @"To"];
   [headers setObject: [self _messageID] forKey: @"Message-Id"];
   message = [NGMimeMessage messageWithHeader: headers];
   [message setBody: content];
-  to = [attendee rfc822Email];
 
   [mailer sendMimePart: message
-          toRecipients: [NSArray arrayWithObject: to]
-                sender: from
+          toRecipients: [NSArray arrayWithObject: [to objectForKey: @"email"]]
+                sender: [to objectForKey: @"email"]
      withAuthenticator: staticAuthenticator inContext: nil];
 }
 
 - (void) _processAlarm: (iCalAlarm *) alarm
              withOwner: (NSString *) ownerId
+      andContainerPath: (NSString *) containerPath
 {
   NGMutableHashMap *headers;
-  NSArray *attendees;
+  NSArray *parts;
   NSData *content, *qpContent;
-  int count, max;
   SOGoMailer *mailer;
-  NSString *from;
+  NSString *subject;
   SOGoUser *owner;
 
+  WOContext *localContext;
+  WOApplication *app;
+  SOGoAptMailReminder *p;
+  NSString *pageName;
+  WOResourceManager *rm;
+
+  SOGoAppointmentFolders *folders;
+  SOGoAppointmentFolder *folder;
+  SOGoUserFolder *userFolder;
+
   owner = [SOGoUser userWithLogin: ownerId];
   mailer = [SOGoMailer mailerWithDomainDefaults: [owner domainDefaults]];
 
-  headers = [self _headersForAlarm: alarm withOwner: owner];
-  content = [[alarm comment] dataUsingEncoding: NSUTF8StringEncoding];
-  qpContent = [content dataByEncodingQuotedPrintable];
-  from = [[owner primaryIdentity] objectForKey: @"email"];
+  localContext = [WOContext context];
+  [localContext setActiveUser: owner];
+  app = [[WOApplication alloc] initWithName: @"SOGo"];
 
-  attendees = [alarm attendees];
-  max = [attendees count];
-  for (count = 0; count < max; count++)
-    [self _sendMessageWithHeaders: headers
-                          content: qpContent
-                       toAttendee: [attendees objectAtIndex: count]
-                             from: from
-                       withMailer: mailer];
+  rm = [[WEResourceManager alloc] init];
+  [app setResourceManager: rm];
+  [rm release];
+  [app _setCurrentContext:localContext];
+
+  userFolder = [[localContext activeUser] homeFolderInContext: localContext ];
+  folders = [userFolder privateCalendars: @"Calendar"
+                               inContext: localContext];
+
+  pageName = [NSString stringWithFormat: @"SOGoAptMail%@", @"Reminder"];
+
+  p = [app pageWithName: pageName inContext: localContext];
+
+  parts = [containerPath componentsSeparatedByString: @"/"];
+  if ([parts count] > 4)
+    {
+      folder = [folders lookupName: [parts objectAtIndex: 4]
+                         inContext: localContext
+                           acquire: NO];
+      [p setCalendarName: [folder displayName]];
+    }
+
+  [p setApt: [alarm parent]];
+  [p setAttendees: [[alarm parent] attendees]];
+
+  content = [[p getBody] dataUsingEncoding: NSUTF8StringEncoding];
+  subject = [p getSubject];
+
+  headers = [self _headersForAlarm: alarm withOwner: owner withSubject: subject];
+  qpContent = [content dataByEncodingQuotedPrintable];
+  [self _sendMessageWithHeaders: headers
+                        content: qpContent
+                             to: [owner primaryIdentity]
+                     withMailer: mailer];
 }
 
 - (void) usage
@@ -191,6 +232,8 @@
 
   int count, max;
 
+  [[SOGoProductLoader productLoader] loadAllProducts: NO];
+
   if ([[NSUserDefaults standardUserDefaults] stringForKey: @"h"])
     {
       [self usage];
@@ -209,9 +252,6 @@
       [staticAuthenticator retain];
     }
 
-  [[SOGoProductLoader productLoader]
-    loadProducts: [NSArray arrayWithObject: @"Appointments.SOGo"]];
-
   eaMgr = [NSClassFromString (@"SOGoEMailAlarmsManager")
                              sharedEMailAlarmsManager];
 
@@ -225,10 +265,12 @@
                                                     second: 0]
                                    toDate: toDate
                                withMetadata: metadata];
+
   max = [alarms count];
   for (count = 0; count < max; count++)
     [self _processAlarm: [alarms objectAtIndex: count]
-              withOwner: [[metadata objectAtIndex: count] objectForKey: @"owner"]];
+              withOwner: [[metadata objectAtIndex: count] objectForKey: @"owner"]
+       andContainerPath: [[[metadata objectAtIndex: count] objectForKey: @"record"] objectForKey: @"c_path"]];
 
   // We now update the next alarm date (if any, for recurring
   // events or tasks for example). This will also delete any emai
