@@ -1,8 +1,6 @@
 /* SOGoToolExpireUserSessions.m - this file is part of SOGo
  *
- * Copyright (C) 2012 Inverse inc.
- *
- * Author: Jean Raby <jraby@inverse.ca>
+ * Copyright (C) 2012-2017 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,10 +40,6 @@
 
 @implementation SOGoToolExpireUserSessions
 
-+ (void) initialize
-{
-}
-
 + (NSString *) command
 {
   return @"expire-sessions";
@@ -53,7 +47,7 @@
 
 + (NSString *) description
 {
-  return @"Expires user sessions without activity for specified number of minutes";
+  return @"expires user sessions without activity for specified number of minutes";
 }
 
 - (void) usage
