@@ -1,6 +1,6 @@
 /* UIxMailPartHTMLViewer.m - this file is part of SOGo
  *
- * Copyright (C) 2007-2013 Inverse inc.
+ * Copyright (C) 2007-2017 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,10 +25,14 @@
 #import <SaxObjC/SaxXMLReaderFactory.h>
 #import <NGExtensions/NSString+misc.h>
 #import <NGExtensions/NSString+Encoding.h>
+#import <NGMime/NGMimeBodyPart.h>
+#import <NGMime/NGMimeType.h>
 
 #include <libxml/encoding.h>
 
 #import <SoObjects/SOGo/NSString+Utilities.h>
+#import <SoObjects/Mailer/NSData+Mail.h>
+#import <SoObjects/Mailer/SOGoMailAccounts.h>
 #import <SoObjects/Mailer/SOGoMailObject.h>
 #import <SoObjects/Mailer/SOGoMailBodyPart.h>
 
@@ -103,197 +107,6 @@ _xmlCharsetForCharset (NSString *charset)
   return encoding;
 }
 
-//
-// In order to avoid a libxml bug/limitation, we strip the charset= parameter
-// to avoid libxml to consider the charset= parameter while it works in UTF-8
-// internally, all the time.
-//
-// A fix was commited by Daniel Veillard following discussions Inverse had
-// with him on the issue:
-//
-// commit a1bc2f2ba4b5317885205d4f71c7c4b1c99ec870
-// Author: Daniel Veillard <veillard redhat com>
-// Date:   Mon May 16 16:03:50 2011 +0800
-//
-//     Add options to ignore the internal encoding
-//  
-//     For both XML and HTML, the document can provide an encoding
-//     either in XMLDecl in XML, or as a meta element in HTML head.
-//     This adds options to ignore those encodings if the encoding
-//     is known in advace for example if the content had been converted
-//     before being passed to the parser.
-//  
-//     * parser.c include/libxml/parser.h: add XML_PARSE_IGNORE_ENC option
-//       for XML parsing
-//     * include/libxml/HTMLparser.h HTMLparser.c: adds the
-//       HTML_PARSE_IGNORE_ENC for HTML parsing
-//     * HTMLtree.c: fix the handling of saving when an unknown encoding is
-//       defined in meta document header
-//     * xmllint.c: add a --noenc option to activate the new parser options
-//
-// 
-static NSData* _sanitizeContent(NSData *theData)
-{
-  NSMutableData *d;
-  NSString *found_tag, *tag;
-  NSEnumerator *tags;
-  const char *bytes;
-  char *buf;
-  int i, j, len;
-  BOOL found_delimiter, in_meta;
-
-  d = [NSMutableData dataWithData: theData];
-  bytes = [d bytes];
-  len = [d length];
-  i = 0;
-
-  in_meta = NO;
-
-  while (i < len)
-    {
-      // We check if we see <meta ...> in which case, we substitute de charset= stuff.
-      if (i < len-5)
-	{
-	  if ((*bytes == '<') &&
-	      (*(bytes+1) == 'm' || *(bytes+1) == 'M') &&
-	      (*(bytes+2) == 'e' || *(bytes+2) == 'E') &&
-	      (*(bytes+3) == 't' || *(bytes+3) == 'T') &&
-	      (*(bytes+4) == 'a' || *(bytes+4) == 'A') &&
-	      (*(bytes+5) == ' '))
-            in_meta = YES;
-	}
-      
-      // We search for something like :
-      // 
-      // <meta http-equiv="Content-Type" content="text/html; charset=Windows-1252">
-      //
-      if (in_meta && i < len-9)
-	{
-	  if ((*bytes == 'c' || *bytes == 'C') &&
-	      (*(bytes+1) == 'h' || *(bytes+1) == 'H') &&
-	      (*(bytes+2) == 'a' || *(bytes+2) == 'A') &&
-	      (*(bytes+3) == 'r' || *(bytes+3) == 'R') &&
-	      (*(bytes+4) == 's' || *(bytes+4) == 'S') &&
-	      (*(bytes+5) == 'e' || *(bytes+5) == 'E') &&
-	      (*(bytes+6) == 't' || *(bytes+6) == 'T') &&
-	      (*(bytes+7) == '='))
-	    {
-	      // We search until we find a '"' or a space
-	      j = 8;
-              found_delimiter = YES;
-
-	      while (*(bytes+j) != ' ' && *(bytes+j) != '"' && *(bytes+j) != '\'')
-		{
-		  j++;
-		  
-		  // We haven't found anything, let's return the data untouched
-		  if ((i+j) >= len)
-                    {
-                      in_meta = found_delimiter = NO;
-                      break;
-                    }
-		}
-
-              if (found_delimiter)
-                {
-                  [d replaceBytesInRange: NSMakeRange(i, j)
-                               withBytes: NULL
-                                  length: 0];
-                  in_meta = found_delimiter = NO;
-                }
-	    }
-	}
-
-      bytes++;
-      i++;
-    }
-
-  /*
-   * Replace badly formatted void tags
-   *
-   * A void tag that begins with a slash is considered invalid.
-   * We remove the slash from those tags.
-   *
-   * Ex: </br> is replaced by <br>
-   */
-
-  if (!VoidTags)
-    {
-      /* see http://www.w3.org/TR/html4/index/elements.html */
-      VoidTags = [[NSArray alloc] initWithObjects: @"area", @"base",
-                                  @"basefont", @"br", @"col", @"frame", @"hr",
-                                  @"img", @"input", @"isindex", @"link",
-                                  @"meta", @"param", @"", nil];
-    }
-
-  bytes = [d bytes];
-  len = [d length];
-  i = 0;
-  while (i < len)
-    {
-      if (i < len-3)
-	{
-          // Search for ending tags
-	  if ((*bytes == '<') && (*(bytes+1) == '/'))
-            {
-              i += 2;
-              bytes += 2;
-              j = 0;
-              found_delimiter = YES;
-
-              while (*(bytes+j) != '>')
-                {
-                  j++;
-                  if ((i+j) >= len)
-                    {
-                      found_delimiter = NO;
-                      break;
-                    }
-                }
-
-              if (found_delimiter && j > 0)
-                {
-                  // Copy the ending tag to a NSString
-                  buf = malloc((j+1) * sizeof(char));
-                  memset (buf, 0, j+1);
-                  memcpy (buf, bytes, j);
-                  found_tag = [NSString stringWithCString: buf encoding: NSUTF8StringEncoding];
-                  
-                  tags = [VoidTags objectEnumerator];
-                  tag = [tags nextObject];
-                  while (tag && found_tag)
-                    {
-                      if ([tag caseInsensitiveCompare: found_tag] == NSOrderedSame)
-                        {
-                          // Remove the leading slash
-                          //NSLog(@"Found void tag with invalid leading slash: </%@>", found_tag);
-                          i--;
-                          [d replaceBytesInRange: NSMakeRange(i, 1)
-                                       withBytes: NULL
-                                          length: 0];
-                          bytes = [d bytes];
-                          bytes += i;
-                          len = [d length];
-                          break;
-                        }
-                      tag = [tags nextObject];
-                    }
-                  free(buf);
-
-                  // Continue the parsing after end tag
-                  i += j;
-                  bytes += j;
-                }
-            }
-        }
-
-      bytes++;
-      i++;
-    }
-  
-  return d;
-}
-
 @interface _UIxHTMLMailContentHandler : NSObject <SaxContentHandler, SaxLexicalHandler>
 {
   NSMutableString *result;
@@ -442,29 +255,96 @@ static NSData* _sanitizeContent(NSData *theData)
 - (void) _appendStyle: (unichar *) _chars
                length: (NSUInteger) _len
 {
-  NSMutableString *declaration;
-  NSUInteger count, length;
-  unichar *start, *currentChar;
+  NSMutableString *sanitizedStyle, *declaration, *rule;
+  NSUInteger count, length, max;
+  unichar *sanitizedChars, *start, *currentChar;
+  BOOL inComment;
 
+  /**
+   * Sanitize style
+   *   - remove control characters
+   *   - remove HTML comment delimiters
+   *   - remove CSS comments
+   */
+  sanitizedStyle = [NSMutableString string];
+  inComment = NO;
   start = _chars;
-  while (*start < 33)
-    start++;
-
-  currentChar = start;
   for (count = 0; count < _len; count++)
     {
       currentChar = _chars + count;
+      if (*currentChar < 32)
+        {
+          // Ignore control characters
+          if (!inComment && currentChar > start)
+            [sanitizedStyle appendString: [NSString stringWithCharacters: start
+                                                                  length: (currentChar - start)]];
+          start = currentChar + 1;
+        }
+      else
+        {
+          if ((currentChar < _chars + _len - 3) &&
+              *currentChar     == '<' &&
+              *(currentChar+1) == '!' &&
+              *(currentChar+2) == '-' &&
+              *(currentChar+3) == '-')
+            {
+              // Ignore starting HTML comment
+              if (!inComment && currentChar > start)
+                [sanitizedStyle appendString: [NSString stringWithCharacters: start
+                                                                      length: (currentChar - start)]];
+              start = currentChar + 4;
+            }
+          else if ((currentChar < _chars + _len - 2) &&
+                   *currentChar     == '-' &&
+                   *(currentChar+1) == '-' &&
+                   *(currentChar+2) == '>')
+            {
+              // Ignore ending HTML comment
+              if (!inComment && currentChar > start)
+                [sanitizedStyle appendString: [NSString stringWithCharacters: start
+                                                                      length: (currentChar - start)]];
+              start = currentChar + 3;
+            }
+          if (currentChar < _chars + _len - 1)
+            {
+              // Ignore CSS comments
+              if (*currentChar == '/' && *(currentChar+1) == '*')
+                {
+                  inComment = YES;
+                  if (currentChar > start)
+                    [sanitizedStyle appendString: [NSString stringWithCharacters: start
+                                                                          length: (currentChar - start)]];
+                }
+              else if (*currentChar == '*' && *(currentChar+1) == '/')
+                {
+                  inComment = NO;
+                  start = currentChar + 2;
+                }
+            }
+        }
+    }
+  if (!inComment && currentChar > start)
+    [sanitizedStyle appendString: [NSString stringWithCharacters: start
+                                               length: (currentChar - start)]];
+
+  /**
+   * Parse sanitized style
+   *   - remove at-rule definitions
+   *   - add custom class to selectors
+   *   - add !important suffix to all rules
+   */
+  rule = [NSMutableString string];
+  max = [sanitizedStyle length];
+  sanitizedChars = NSZoneMalloc (NULL, max * sizeof (unichar));
+  [sanitizedStyle getCharacters: sanitizedChars];
+  start = sanitizedChars;
+  currentChar = start;
+  for (count = 0; count < max; count++)
+    {
+      currentChar = sanitizedChars + count;
       if (inCSSDeclaration)
         {
-          if (*currentChar < 32)
-            {
-              // Append substring since last valid character and reset start counter
-              if (currentChar > start)
-                [declaration appendString: [NSString stringWithCharacters: start
-                                                                   length: (currentChar - start)]];
-              start = currentChar + 1;
-            }
-          else if (*currentChar == '}')
+          if (*currentChar == '}')
             {
               inCSSDeclaration = NO;
               if (hasEmbeddedCSS)
@@ -475,17 +355,17 @@ static NSData* _sanitizeContent(NSData *theData)
                 }
               else
                 {
-                  // Prefix CSS rule
-                  length = (currentChar - start);
+                  // Prefix CSS rule including ending curly bracket
+                  length = (currentChar - start) + 1;
                   [declaration appendString: [NSString stringWithCharacters: start length: length]];
-                  [css appendFormat: @".SOGoHTMLMail-CSS-Delimiter %@\n", declaration];
-                  start = currentChar;
+                  [css appendString: declaration];
+                  start = currentChar + 1;
                 }
             }
           else if (*currentChar == ';')
             {
               // Add !important
-              if ((currentChar < _chars - 10) ||
+              if ((currentChar < sanitizedChars + 10) ||
                   !((*(currentChar-1) == 't' || *(currentChar-1) == 'T') &&
                     (*(currentChar-2) == 'n' || *(currentChar-2) == 'N') &&
                     (*(currentChar-3) == 'a' || *(currentChar-3) == 'A') &&
@@ -506,48 +386,49 @@ static NSData* _sanitizeContent(NSData *theData)
         }
       else
         {
-          if (*currentChar < 32)
+          if (*currentChar == '{')
             {
-              // Append substring since last valid character and reset start counter
-              if (currentChar > start)
-                [css appendString: [NSString stringWithCharacters: start
-                                                           length: (currentChar - start)]];
+              // Start of rule declaration
+              inCSSDeclaration = YES;
+              if (!hasEmbeddedCSS)
+                {
+                  length = (currentChar - start);
+                  [rule appendFormat: @".SOGoHTMLMail-CSS-Delimiter %@ {",
+                        [NSString stringWithCharacters: start length: length]];
+                  [css appendString: rule];
+                }
+              rule = [NSMutableString string];
+              declaration = [NSMutableString string];
               start = currentChar + 1;
             }
-          else
+          if (*currentChar == '}')
             {
-              if (*currentChar == '{')
+              // CSS syntax error: ending declaration character while not in a CSS declaration.
+              // Ignore eveything from last CSS declaration.
+              start = currentChar + 1;
+              rule = [NSMutableString string];
+            }
+          else if (hasEmbeddedCSS)
+            {
+              if (*currentChar == ';')
                 {
-                  // Start of rule declaration
-                  inCSSDeclaration = YES;
-                  declaration = [NSMutableString string];
-                }
-              if (*currentChar == '}')
-                // CSS syntax error: ending declaration character while not in a CSS declaration.
-                // Ignore eveything from last CSS declaration.
-                start = currentChar + 1;
-              else if (hasEmbeddedCSS)
-                {
-                  if (*currentChar == ';')
-                    {
-                      // End of at-rule definition; remove it from the stylesheet
-                      hasEmbeddedCSS = NO;
-                      start = currentChar + 1;
-                    }
-                }
-              else if (*currentChar == ',')
-                {
-                  // Prefix CSS selector
-                  length = (currentChar - start);
-                  [css appendFormat: @" .SOGoHTMLMail-CSS-Delimiter %@,",
-                       [NSString stringWithCharacters: start length: length]];
+                  // End of at-rule definition; remove it from the stylesheet
+                  hasEmbeddedCSS = NO;
                   start = currentChar + 1;
                 }
-              else if (*currentChar == '@')
-                {
-                  // Start of at-rule definition
-                  hasEmbeddedCSS = YES;
-                }
+            }
+          else if (*currentChar == ',')
+            {
+              // Prefix CSS selector
+              length = (currentChar - start);
+              [rule appendFormat: @" .SOGoHTMLMail-CSS-Delimiter %@,",
+                    [NSString stringWithCharacters: start length: length]];
+              start = currentChar + 1;
+            }
+          else if (*currentChar == '@')
+            {
+              // Start of at-rule definition
+              hasEmbeddedCSS = YES;
             }
         }
     }
@@ -686,7 +567,8 @@ static NSData* _sanitizeContent(NSData *theData)
               
               if (!skipAttribute)
                 [resultPart appendFormat: @" %@=\"%@\"",
-                            name, [value safeStringByEscapingXMLString: NO]];
+                            name, [value stringByReplacingString: @"\""
+                                                  withString: @""]];
             }
           
           if ([VoidTags containsObject: lowerName])
@@ -741,7 +623,9 @@ static NSData* _sanitizeContent(NSData *theData)
         {
           if ([lowerName isEqualToString: @"body"])
             {
-              inBody = NO;
+              // Assume <body> never ends to properly display incorrectly constructed messages.
+              // See bug #4492
+              // inBody = NO;
               if (css)
                 [self _finishCSS];
             }
@@ -908,14 +792,15 @@ static NSData* _sanitizeContent(NSData *theData)
 {
   NSObject <SaxXMLReader> *parser;
   NSData *preparsedContent;
-  SOGoMailObject *mail;
   NSString *s;
 
   xmlCharEncoding enc;
 
-  mail = [self clientObject];
+  if ([[self decodedFlatContent] isKindOfClass: [NGMimeBodyPart class]])
+    preparsedContent = [[[self decodedFlatContent] body] sanitizedContentUsingVoidTags: VoidTags];
+  else
+    preparsedContent = [[self decodedFlatContent] sanitizedContentUsingVoidTags: VoidTags];
 
-  preparsedContent = _sanitizeContent([super decodedFlatContent]);
   parser = [[SaxXMLReaderFactory standardXMLReaderFactory]
              createXMLReaderForMimeType: @"text/html"];
 
@@ -1031,18 +916,25 @@ static NSData* _sanitizeContent(NSData *theData)
 {
   NSObject <SaxXMLReader> *parser;
   NSData *preparsedContent;
-  SOGoMailObject *mail;
   SOGoMailBodyPart *part;
   NSString *encoding;
   xmlCharEncoding enc;
 
-  part = [self clientObject];
-  mail = [part mailObject];
-
-  preparsedContent = _sanitizeContent([part fetchBLOB]);
   parser = [[SaxXMLReaderFactory standardXMLReaderFactory]
              createXMLReaderForMimeType: @"text/html"];
-  encoding = [[part partInfo] valueForKey: @"encoding"];
+
+  if ([[self decodedFlatContent] isKindOfClass: [NGMimeBodyPart class]])
+    {
+      preparsedContent = [[[self decodedFlatContent] body] sanitizedContentUsingVoidTags: VoidTags];
+      encoding = [[[self decodedFlatContent] contentType] valueOfParameter: @"charset"];
+    }
+  else
+    {
+      part = [self clientObject];
+      preparsedContent = [[part fetchBLOB] sanitizedContentUsingVoidTags: VoidTags];
+      encoding = [[part partInfo] valueForKey: @"encoding"];
+    }
+
   if (![encoding length])
     encoding = @"us-ascii";
 
