@@ -1,6 +1,6 @@
 /* UIxMailActions.m - this file is part of SOGo
  *
- * Copyright (C) 2007-2013 Inverse inc.
+ * Copyright (C) 2007-2016 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,27 +18,25 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#import <Foundation/NSArray.h>
-#import <Foundation/NSDictionary.h>
-#import <Foundation/NSString.h>
+#import <Foundation/NSValue.h>
 
-#import <NGObjWeb/WOContext.h>
 #import <NGObjWeb/WOContext+SoObjects.h>
 #import <NGObjWeb/WORequest.h>
-#import <NGObjWeb/WOResponse.h>
 #import <NGObjWeb/NSException+HTTP.h>
 
+#import <SoObjects/Mailer/NSString+Mail.h>
 #import <SoObjects/Mailer/SOGoDraftObject.h>
 #import <SoObjects/Mailer/SOGoDraftsFolder.h>
 #import <SoObjects/Mailer/SOGoMailAccount.h>
 #import <SoObjects/Mailer/SOGoMailObject.h>
+#import <SoObjects/Mailer/SOGoMailObject+Draft.h>
+#import <SoObjects/SOGo/NSArray+Utilities.h>
+#import <SoObjects/SOGo/NSDictionary+Utilities.h>
 #import <SoObjects/SOGo/NSString+Utilities.h>
 #import <SoObjects/SOGo/SOGoUser.h>
 #import <SoObjects/SOGo/SOGoUserSettings.h>
 #import <SoObjects/SOGo/SOGoUserDefaults.h>
 
-#import "../Common/WODirectAction+SOGo.h"
-
 #import "UIxMailActions.h"
 
 @implementation UIxMailActions
@@ -47,20 +45,29 @@
 {
   SOGoMailAccount *account;
   SOGoMailObject *co;
-  SOGoDraftsFolder *folder;
+  SOGoDraftsFolder *drafts;
   SOGoDraftObject *newMail;
-  NSString *newLocation;
+  NSString *accountName, *mailboxName, *messageName;
+  NSDictionary *data;
 
   co = [self clientObject];
   account = [co mailAccountFolder];
-  folder = [account draftsFolderInContext: context];
-  newMail = [folder newDraft];
+  drafts = [account draftsFolderInContext: context];
+  newMail = [drafts newDraft];
   [newMail fetchMailForReplying: co toAll: toAll];
 
-  newLocation = [NSString stringWithFormat: @"%@/edit",
-			  [newMail baseURLInContext: context]];
+  accountName = [account nameInContainer];
+  mailboxName = [drafts absoluteImap4Name];
+  mailboxName = [mailboxName substringWithRange: NSMakeRange(1, [mailboxName length] -2)];
+  messageName = [newMail nameInContainer];
 
-  return [self redirectToLocation: newLocation];
+  data = [NSDictionary dictionaryWithObjectsAndKeys:
+                         accountName, @"accountId",
+                       mailboxName, @"mailboxPath",
+                       messageName, @"draftId", nil];
+
+  return [self responseWithStatus: 201
+                        andString: [data jsonRepresentation]];
 }
 
 - (WOResponse *) replyAction
@@ -77,26 +84,92 @@
 {
   SOGoMailAccount *account;
   SOGoMailObject *co;
-  SOGoDraftsFolder *folder;
+  SOGoDraftsFolder *drafts;
   SOGoDraftObject *newMail;
   SOGoUserDefaults *ud;
-  NSString *newLocation;
+  NSString *accountName, *mailboxName, *messageName;
+  NSDictionary *data;
   BOOL htmlComposition;
 
   co = [self clientObject];
   account = [co mailAccountFolder];
-  folder = [account draftsFolderInContext: context];
-  newMail = [folder newDraft];
+  drafts = [account draftsFolderInContext: context];
+  newMail = [drafts newDraft];
   ud = [[context activeUser] userDefaults];
   htmlComposition = [[ud mailComposeMessageType] isEqualToString: @"html"];
 
   [newMail setIsHTML: htmlComposition];
   [newMail fetchMailForForwarding: co];
 
-  newLocation = [NSString stringWithFormat: @"%@/edit",
-			  [newMail baseURLInContext: context]];
+  accountName = [account nameInContainer];
+  mailboxName = [drafts absoluteImap4Name];
+  mailboxName = [mailboxName substringWithRange: NSMakeRange(1, [mailboxName length] -2)];
+  messageName = [newMail nameInContainer];
 
-  return [self redirectToLocation: newLocation];
+  data = [NSDictionary dictionaryWithObjectsAndKeys:
+                         accountName, @"accountId",
+                       mailboxName, @"mailboxPath",
+                       messageName, @"draftId",
+                       // Message was saved ([SOGoDraftObject fetchMailForForwarding:]) so IMAP ID exists
+                       [NSNumber numberWithInt: [newMail IMAP4ID]], @"uid", nil];
+
+  return [self responseWithStatus: 201
+                        andString: [data jsonRepresentation]];
+}
+
+- (WOResponse *) viewPlainAction
+{
+  BOOL htmlContent;
+  NSArray *acceptedTypes, *types;
+  NSDictionary *parts;
+  NSMutableArray *keys;
+  NSMutableDictionary *data;
+  NSString *rawPart, *contentKey, *subject, *content;
+  NSUInteger index;
+  SOGoMailObject *co;
+
+  co = [self clientObject];
+  subject = [co decodedSubject];
+  htmlContent = NO;
+  data = [NSMutableDictionary dictionary];
+
+  if (subject)
+    [data setObject: subject
+             forKey: @"subject"];
+
+  // Fetch the text parts of the message body structure
+  acceptedTypes = [NSArray arrayWithObjects: @"text/plain", @"text/html", nil];
+  keys = [NSMutableArray array];
+  [co addRequiredKeysOfStructure: [co bodyStructure]
+                              path: @"" toArray: keys acceptedTypes: acceptedTypes
+                          withPeek: NO];
+
+  // Use plain part if available, otherwise use the HTML part
+  types = [keys objectsForKey: @"mimeType" notFoundMarker: @""];
+  index = [types indexOfObject: @"text/plain"];
+  if (index == NSNotFound)
+    {
+      index = [types indexOfObject: @"text/html"];
+      htmlContent = YES;
+    }
+
+  // Fetch part and convert HTML if necessary
+  contentKey = [keys objectAtIndex: index];
+  parts = [co fetchPlainTextStrings: [NSArray arrayWithObject: contentKey]];
+  if ([parts count] > 0)
+    {
+      rawPart = [[parts allValues] objectAtIndex: 0];
+      if (htmlContent)
+        content = [rawPart htmlToText];
+      else
+        content = rawPart;
+      if (content)
+        [data setObject: [content stringByTrimmingSpaces]
+                 forKey: @"content"];
+    }
+
+  return [self responseWithStatus: 201
+                        andString: [data jsonRepresentation]];
 }
 
 /* active message */
@@ -148,7 +221,6 @@
 
 - (void) collapseAction: (BOOL) isCollapsing
 {
-  SOGoMailObject *co;
   WORequest *request;
   NSMutableDictionary *moduleSettings, *threadsCollapsed, *content;
   NSMutableArray *mailboxThreadsCollapsed;
@@ -160,10 +232,9 @@
   keyForMsgUIDs = [content objectForKey:@"currentMailbox"];
   msguid = [content objectForKey:@"msguid"];
 
-  co = [self clientObject];
   us = [[context activeUser] userSettings];
   if (!(moduleSettings = [us objectForKey: @"Mail"]))
-    [us setObject:[NSMutableDictionary dictionnary] forKey: @"Mail"];
+    [us setObject:[NSMutableDictionary dictionary] forKey: @"Mail"];
 
   if (isCollapsing)
     {
@@ -223,13 +294,14 @@
 }
 
 /* SOGoDraftObject */
-- (WOResponse *) editAction
+- (id <WOActionResults>) editAction
 {
+  id <WOActionResults> response;
   SOGoMailAccount *account;
   SOGoMailObject *co;
   SOGoDraftsFolder *folder;
   SOGoDraftObject *newMail;
-  NSString *newLocation;
+  NSDictionary *data;
 
   co = [self clientObject];
   account = [co mailAccountFolder];
@@ -238,10 +310,13 @@
   [newMail fetchMailForEditing: co];
   [newMail storeInfo];
 
-  newLocation = [NSString stringWithFormat: @"%@/edit",
-			  [newMail baseURLInContext: context]];
+  data = [NSDictionary dictionaryWithObject: [newMail nameInContainer]
+                                     forKey: @"draftId"];
 
-  return [self redirectToLocation: newLocation];
+  response = [self responseWithStatus: 200
+                            andString: [data jsonRepresentation]];
+
+  return response;
 }
 
 - (id) deleteAction
