@@ -272,7 +272,7 @@
     }
 
   if (rememberLogin)
-    [response addCookie: [self _cookieWithUsername: username]];
+    [response addCookie: [self _cookieWithUsername: [params objectForKey: @"userName"]]];
   else
     [response addCookie: [self _cookieWithUsername: nil]];
 
@@ -535,6 +535,11 @@
   return [[context resourceLookupLanguages] objectAtIndex: 0];
 }
 
+- (NSString *) localizedLanguage
+{
+  return [self labelForKey: [self language]];
+}
+
 - (NSArray *) languages
 {
   return [[SOGoSystemDefaults sharedSystemDefaults] supportedLanguages];
@@ -618,6 +623,7 @@
       creds = [auth parseCredentials: [authCookie value]];
       xsrfCookie = [WOCookie cookieWithName: @"XSRF-TOKEN"
                                       value: [[SOGoSession valueForSessionKey: [creds lastObject]] asSHA1String]];
+      [xsrfCookie setPath: [NSString stringWithFormat: @"/%@/", [request applicationName]]];
       [response addCookie: xsrfCookie];
     }
   else
