@@ -18,7 +18,7 @@ module.exports = function(grunt) {
   };
   var custom_vendor_files = {
     'js/vendor/angular-file-upload.min.js': ['bower_components/angular-file-upload/dist/angular-file-upload.js', 'js/Common/angular-file-upload.trump.js'],
-    'js/vendor/FileSaver.min.js': ['bower_components/file-saver.js/FileSaver.js']
+    'js/vendor/FileSaver.min.js': ['bower_components/FileSaver/dist/FileSaver.js']
   };
 
   require('time-grunt')(grunt);
