@@ -11,8 +11,8 @@
   /**
    * @ngInject
    */
-  configure.$inject = ['$stateProvider', '$urlServiceProvider'];
-  function configure($stateProvider, $urlServiceProvider) {
+  configure.$inject = ['$stateProvider', '$urlServiceProvider', '$mdThemingProvider'];
+  function configure($stateProvider, $urlServiceProvider, $mdThemingProvider) {
     $stateProvider
       .state('calendars', {
         url: '/calendar',
